from datetime import datetime
from typing import List

from banyan.api.base import ServiceBase, Resource
from banyan.model import BanyanApiObject
from banyan.model.event_v2 import EventV2, EventOrID


class EventV2API(ServiceBase):
    class Meta:
        data_class = EventV2
        info_class = EventV2
        arg_type = EventOrID
        list_uri = '/events'
        uri_param = 'EventID'
        obj_name = 'event'
        supports_paging = False

    def create(self, obj: BanyanApiObject) -> str:
        raise NotImplementedError('The Banyan API does not support this operation')

    def update(self, obj: BanyanApiObject) -> str:
        raise NotImplementedError('The Banyan API does not support this operation')

    def delete(self, obj: BanyanApiObject) -> str:
        raise NotImplementedError('The Banyan API does not support this operation')

    def list(self, before_dt: datetime = None, after_dt: datetime = None, order: str = None,
             event_type: str = None, subtype: str = None, action: str = None,
             email_address: str = None, device_id: str = None, device_serial: str = None,
             container_id: str = None, service_name: str = None, event_id: str = None) -> list:
        params = ServiceBase.args_to_html_params([
            (before_dt, 'before', lambda: int(before_dt.timestamp() * 1000)),
            (after_dt, 'after', lambda: int(after_dt.timestamp() * 1000)),
            (order, 'order', order),
            (event_type, 'type', event_type),
            (subtype, 'sub_type', subtype),
            (action, 'action', action),
            (email_address, 'user_email', email_address),
            (device_id, 'device_id', device_id),
            (device_serial, 'serialnumber', device_serial),
            (container_id, 'workload_container_id', container_id),
            (service_name, 'service_name', service_name),
            (event_id, 'id', event_id),
        ])
        response_json = self._client.api_request('GET', self.Meta.list_uri, params=params)
        response_json = response_json['data']
        data: List[Resource] = self.Meta.info_class.Schema().load(response_json, many=True)
        self._build_cache(data)
        return data



from datetime import datetime
from typing import List

from cement import Controller, ex

from banyan.api.event_v2 import EventV2API
from banyan.model.event_v2 import EventV2, EventV2Type, EventV2Subtype


class EventV2Controller(Controller):
    class Meta:
        label = 'event'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'report on security events'

    @property
    def _client(self) -> EventV2API:
        return self.app.client.events

    @ex(help='list events',
        arguments=[
            (['--type'],
             {
                 'help': 'Filters events of one event type.',
                 'choices': EventV2Type.choices(),
             }),
            (['--subtype'],
             {
                 'help': 'Filters events of one subtype (conditionally depends on type).',
                 'choices': EventV2Subtype.choices(),
             }),
            (['--action'],
             {
                 'help': 'Filters events by one of the possible event action values (e.g. "grant", "deny").',
             }),
            (['--email'],
             {
                 'help': 'Filters events associated with a single user, based on their email address.',
             }),
            (['--device-id'],
             {
                 'help': 'Filters events associated with a single device, based on its device ID.',
             }),
            (['--serial-number'],
             {
                 'help': 'Filters events associated with a single device, based on its serial number.',
             }),
            (['--container-id'],
             {
                 'help': 'Filters events associated with a single workload, based on its container ID.',
             }),
            (['--service-name'],
             {
                 'help': 'Filters events associated with a single service, based on its service name.',
             }),
            (['--event-id'],
             {
                 'help': 'Retrieve a single event based on its event ID.',
             }),
            (['--before'],
             {
                 'help': 'Filters events that occurred before a specific time.',
                 'type': datetime.fromisoformat,
             }),
            (['--after'],
             {
                 'help': 'Filters events that occurred after a specific time.',
                 'type': datetime.fromisoformat,
             }),
            (['--order'],
             {
                 'help': 'Sets the order for returned events based on created_at timestamp. Supported values '
                         'ASC, DESC. Default is DESC.',
                 'choices': ('ASC', 'DESC')
             }),
        ]
        )
    def list(self):
        events: List[EventV2] = self._client.list(before_dt=self.app.pargs.before, after_dt=self.app.pargs.after,
                                                  order=self.app.pargs.order, event_type=self.app.pargs.type,
                                                  subtype=self.app.pargs.subtype,
                                                  action=self.app.pargs.action, email_address=self.app.pargs.email,
                                                  device_id=self.app.pargs.device_id,
                                                  device_serial=self.app.pargs.serial_number,
                                                  container_id=self.app.pargs.container_id,
                                                  service_name=self.app.pargs.service_name,
                                                  event_id=self.app.pargs.event_id)
        event_json = EventV2.Schema().dump(events, many=True)
        self.app.render(event_json, handler='json', indent=2, sort_keys=True)



from abc import ABC
from dataclasses import field
from datetime import datetime
from ipaddress import IPv4Interface
from typing import ClassVar, Type, Union

from aenum import StrEnum
from marshmallow import fields, Schema
from marshmallow_dataclass import dataclass
from semver import VersionInfo

from banyan.core.exc import BanyanError

API_VERSION = "rbac.banyanops.com/v1"


class BanyanEnum(StrEnum):

    # noinspection PyTypeChecker
    @classmethod
    def choices(cls):
        return [m.value for m in cls]


class TimestampField(fields.DateTime):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        elif isinstance(value, int):
            return datetime.fromtimestamp(value)
        else:
            return super()._deserialize(value, attr, data, **kwargs)

    def _serialize(self, value: datetime, attr, obj, **kwargs):
        if not value:
            return 0
        elif isinstance(value, datetime):
            return int(value.timestamp())
        else:
            return super()._serialize(value, attr, obj, **kwargs)


class NanoTimestampField(fields.DateTime):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        elif isinstance(value, int):
            return datetime.fromtimestamp(value / 1000000000)
        else:
            return super()._deserialize(value, attr, data, **kwargs)

    def _serialize(self, value: datetime, attr, obj, **kwargs):
        if not value:
            return 0
        elif isinstance(value, datetime):
            return int(value.timestamp() * 1000000000)
        else:
            return super()._serialize(value, attr, obj, **kwargs)


class MilliTimestampField(fields.DateTime):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        elif isinstance(value, int):
            try:
                return datetime.fromtimestamp(value / 1000)
            except ValueError as ex:
                raise ValueError(f'{ex.args[0]}, value = {value}, attr = {attr}')
        else:
            return super()._deserialize(value, attr, data, **kwargs)

    def _serialize(self, value: datetime, attr, obj, **kwargs):
        if not value:
            return 0
        elif isinstance(value, datetime):
            return int(value.timestamp() * 1000)
        else:
            return super()._serialize(value, attr, obj, **kwargs)


class IPv4InterfaceField(fields.Field):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        elif isinstance(value, str):
            return IPv4Interface(value)
        else:
            return super()._deserialize(value, attr, data, **kwargs)

    def _serialize(self, value, attr, data, **kwargs):
        if not value:
            return ""
        elif isinstance(value, IPv4Interface):
            return str(value)
        else:
            return super()._serialize(value, attr, data, **kwargs)


class VersionField(fields.Field):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        elif isinstance(value, str):
            if '_' in value:
                value = value.split('_')[0]
            return VersionInfo.parse(value)
        else:
            return super()._deserialize(value, attr, data, **kwargs)

    def _serialize(self, value, attr, data, **kwargs):
        if not value:
            return ""
        elif isinstance(value, VersionInfo):
            return str(value)
        else:
            return super()._serialize(value, attr, data, **kwargs)


@dataclass
class BanyanApiObject:
    TYPE = "origin"
    apiVersion: str
    kind: str
    type: str
    Schema: ClassVar[Type[Schema]] = Schema

    def __post_init__(self):
        self.apiVersion = API_VERSION
        self.type = self.TYPE

    @property
    def name(self) -> str:
        raise NotImplementedError('not implemented in the base class')


@dataclass
class Resource(ABC):
    Schema: ClassVar[Type[Schema]] = Schema

    @property
    def name(self) -> str:
        raise BanyanError('not implemented')

    @property
    def id(self) -> str:
        raise BanyanError('not implemented')


ResourceOrName = Union[Resource, str]


@dataclass
class InfoBase(Resource, ABC):
    created_by: str = field(metadata={'data_key': 'CreatedBy'})
    created_at: datetime = field(metadata={'marshmallow_field': NanoTimestampField(data_key='CreatedAt')})
    last_updated_by: str = field(metadata={'data_key': 'LastUpdatedBy'})
    last_updated_at: datetime = field(metadata={'marshmallow_field': NanoTimestampField(data_key='LastUpdatedAt')})
    deleted_by: str = field(metadata={'data_key': 'DeletedBy'})
    deleted_at: datetime = field(metadata={'marshmallow_field': NanoTimestampField(data_key='DeletedAt')})
    spec: str
    description: str
    Schema: ClassVar[Type[Schema]] = Schema



from typing import List

from banyan.api.base import ServiceBase
from banyan.core.exc import BanyanError
from banyan.model.user_device import Device, TrustScore


class DeviceAPI(ServiceBase):
    """
    API service for dealing with end-user devices (laptops, tablets, phones, etc).

    """

    # TODO: include name or device_friendly_name?
    _UPDATE_FIELDS = ['architecture', 'model', 'platform', 'ownership', 'is_banned']

    class Meta:
        """
        :meta private:
        """
        data_class = Device
        info_class = Device
        arg_type = str
        list_uri = '/devices'
        delete_uri = '/delete_device'
        insert_uri = '/mdm/update_device'
        uri_param = 'SerialNumber'
        obj_name = 'device'
        supports_paging = True

    def create(self, obj: Device):
        """
        Raises an exception. Devices cannot be created via API. They must be registered by installing the
        Banyan App on a device.

        :param obj: A device which will definitely not be created.
        :type obj: Device
        :raises: :py:exc:`BanyanError`
        """
        raise BanyanError('devices cannot be created via the API')

    def update(self, obj: Device) -> str:
        """
        Updates a limited set of properties belonging to a device. The properties that can be updated are:

        * Device hardware data: :py:data:`architecture`, :py:data:`model`, and :py:data:`platform`.
        * Corporate ownership status: :py:data:`ownership`.
        * Banned status: :py:data:`is_banned`.
        * User-friendly device name: :py:data:`device_friendly_name`.

        Changes to any other fields in the :py:class:`Device` object will be ignored.

        :param obj: The device to be updated.
        :type obj: Device
        :return: Message from the server indicating success or failure.
        :rtype: str
        """
        self._ensure_exists(obj.serial_number)
        response_json = self._client.api_request('POST',
                                                 self.Meta.insert_uri,
                                                 params={self.Meta.uri_param: obj.serial_number},
                                                 json=obj.Schema(only=DeviceAPI._UPDATE_FIELDS).dump(obj))
        return response_json['Message']

    def ban(self, obj: Device) -> str:
        """
        Bans a device, preventing it from accessing sites controlled by Banyan.

        :param obj: The device to be banned.
        :type obj: Device
        :return: Message from the server indicating success or failure.
        :rtype: str
        """
        obj.is_banned = True
        return self.update(obj)

    def unban(self, obj: Device) -> str:
        """
        Un-bans a device, allowing it to resume accessing sites controlled by Banyan.

        .. note:: After a device is unbanned, it must send updated Trust Factors to the server
           before it will be allowed to access anything. This happens automatically about once an hour,
           or can be forced manually by clicking the "Send Device Features" link in the app settings.

        :param obj: The device to be unbanned.
        :type obj: Device
        :return: Message from the server indicating success or failure.
        :rtype: str
        """
        obj.is_banned = False
        return self.update(obj)

    def set_max_trustlevel(self, obj: Device, max_level: str, reason: str, ext_source: str) -> str:
        """
        Overrides the maximum trust score a device will be allowed to have.

        :param obj: The device to be updated.
        :type obj: Device
        :param max_level: One of the :py:class:`TrustLevel` constants (:py:const:`ALWAYS_DENY`,
            :py:const:`LOW`, :py:const:`MEDIUM`, :py:const:`HIGH`, or :py:const:`ALWAYS_ALLOW`).
        :type max_level: str
        :param reason: Message indicating why the trust score was affected. This may be displayed to
            the user inside the app.
        :type reason: str
        :param ext_source: Name of the external source affecting the trust level (e.g. "CrowdStrike",
            "Carbon Black", etc).
        :type ext_source: str
        :return: Message from the server indicating success or failure.
        :rtype: str
        """
        self._ensure_exists(obj.serial_number)
        response_json = self._client.api_request('POST',
                                                 '/set_max_trust_level',
                                                 params={self.Meta.uri_param: obj.serial_number},
                                                 json={
                                                     'Level': max_level,
                                                     'Reason': reason,
                                                     'ExtSource': ext_source
                                                 })
        return TrustScore.Schema().load(response_json)

    def get_trustscores(self, obj: Device) -> List[TrustScore]:
        """
        Returns the detailed list of Trust Factors that make up the device's Trust Score.

        In general, you will want to use the score in the `trust_data` field of the :py:class:`Device`
        object. When additional detail is required, this method may be used instead.

        :param obj: The device to be updated.
        :type obj: Device
        :return: a list of :py:class:`TrustScore` objects. Most devices will have one score
            with a `TrustType` of "Device". If a device's maximum trust score has been limited by
            a previous call to :py:meth:`set_max_trustlevel`, there will be an additional score
            with a `TrustType` of "External".
        :rtype: List[TrustScore]
        """
        self._ensure_exists(obj.serial_number)
        response_json = self._client.api_request('GET',
                                                 '/trustscore',
                                                 params={self.Meta.uri_param: obj.serial_number})
        return TrustScore.Schema().load(response_json, many=True)



"""
The `banyan.api` module contains classes and methods that do the work of actually interacting with the Banyan
Command Center, translating JSON responses into objects from the :py:mod:`banyan.model` module.
"""

import logging
import os
from typing import Dict, Any, List, Union

import requests
from requests.auth import AuthBase

from banyan.api.attachment import AttachmentAPI
from banyan.api.audit import AuditAPI
from banyan.api.device import DeviceAPI
from banyan.api.event_v2 import EventV2API
from banyan.api.netagent import NetagentAPI
from banyan.api.policy import PolicyAPI
from banyan.api.role import RoleAPI
from banyan.api.service import ServiceAPI
from banyan.api.shield import ShieldAPI
from banyan.api.user import UserAPI
from banyan.core.exc import BanyanError

JsonListOrObj = Union[List, Dict]


class BanyanApiClient:
    """
    Main class for interacting with the Banyan API.

    :param api_server_url: URL for the Banyan Command Center. This should be the same as the
        URL you enter in your browser to log into the Command Center. If not supplied, we will
        look for an environment variable named :envvar:`BANYAN_API_URL`; if that is also not present,
        the URL defaults to <https://net.banyanops.com>.
    :type api_server_url: str
    :param refresh_token: Initial API token used to authorize the connection. The refresh token
        will then be exchanged for an access token. If not supplied, we look for an environment
        varialbe named :envvar:`BANYAN_REFRESH_TOKEN`; if that is also not present, it causes a
        :py:exc:`BanyanError`.
    :type refresh_token: str
    :param debug: If True, extra debugging output from the Requests module will be logged.
    :type debug: bool
    :param log: Optional logger to use in debug mode. If not provided, the standard
        logger will be used.
    :type log: logging.Logger
    :raises: :py:exc:`BanyanError` if no refresh token is provided.
    """

    DEFAULT_API_URL = 'https://net.banyanops.com'
    """
    Default API server URL if none is specified and if a :envvar:`BANYAN_API_URL` environment variable
    is not found.
    """

    JSON_TYPE = 'application/json'
    """
    Default MIME content type for all requests and responses.
    """

    class _Auth(AuthBase):
        """
        Internal class for storing API tokens.
        :meta private:
        """

        def __init__(self, parent) -> None:
            self._obj = parent

        def __call__(self, r: requests.Request) -> requests.Request:
            token = self._obj.access_token or self._obj.refresh_token
            r.headers['Authorization'] = f'Bearer {token}'
            return r

    def __init__(self, api_server_url: str = None, refresh_token: str = None, debug: bool = False,
                 log: logging.Logger = None) -> None:
        self._debug = debug
        self._log = log
        self._api_url = self._normalize_url(api_server_url or os.getenv('BANYAN_API_URL')
                                            or BanyanApiClient.DEFAULT_API_URL)
        self._access_token = None
        self._refresh_token = refresh_token or os.getenv('BANYAN_REFRESH_TOKEN')
        if not self._refresh_token:
            raise BanyanError("Refresh token must be set")
        if self._debug:
            requests_log = logging.getLogger('requests.packages.urllib3')
            requests_log.setLevel(logging.DEBUG)
            requests_log.propagate = True
        self._http = self._create_session()
        self._services = ServiceAPI(self)
        self._policies = PolicyAPI(self)
        self._attach = AttachmentAPI(self)
        self._roles = RoleAPI(self)
        self._shields = ShieldAPI(self)
        self._agents = NetagentAPI(self)
        self._users = UserAPI(self)
        self._devices = DeviceAPI(self)
        self._events = EventV2API(self)
        self._audit = AuditAPI(self)

    # noinspection PyMethodMayBeStatic
    def _normalize_url(self, url: str) -> str:
        if '/api' not in url:
            url += '/api/v1'
        return url

    def _create_session(self) -> requests.Session:
        http = requests.session()
        http.headers['Accept'] = 'application/json'
        http.auth = self._Auth(self)
        if self._debug:
            requests_log = logging.getLogger("requests.packages.urllib3")
            requests_log.setLevel(logging.DEBUG)
            requests_log.propagate = True
        return http

    def get_access_token(self) -> str:
        """
        Forces the client to obtain a new access token. This method is provided for testing. You should not need
        to call it, as the client will automatically fetch an access token when needed.

        :return: the new access token.
        :raises: :py:exc:`BanyanError` if the refresh token is invalid.
        """
        content = self._request('POST', '/refresh_token').json()
        self._access_token = content['Message']
        return self._access_token

    def _request(self, method: str, url: str, params: Dict[str, Any] = None, data: Any = None,
                 headers: Dict[str, str] = None, cookies: Dict[str, str] = None,
                 files=None, auth=None, timeout=None, allow_redirects=True, proxies=None,
                 hooks=None, stream=None, verify=None, cert=None, json=None) -> requests.Response:
        if '://' not in url:
            url = self._api_url + url
        response = self._http.request(method, url, params, data, headers, cookies, files, auth or self._http.auth,
                                      timeout, allow_redirects, proxies, hooks, stream, verify, cert, json)
        # logging.debug(response.content)
        if response.status_code >= 400:
            try:
                content = response.json()
                if 'Message' in content:
                    raise BanyanError(f'{response.status_code} Client Error: {response.reason} for '
                                      f'url: {response.url}: {content["Message"]}')
                elif 'error' in content:
                    raise BanyanError(f'{response.status_code} Client Error: {response.reason} for '
                                      f'url: {response.url}: {content["error_code"]}: {content["error"]}')
            except ValueError:
                raise BanyanError(f'{response.status_code} Client Error: {response.reason} for url: {response.url}')
        return response

    def api_request(self, method: str, uri: str, params: Dict[str, Any] = None, data: Any = None,
                    json: str = None, headers: Dict[str, str] = None, accept: str = None) -> JsonListOrObj:
        """
        Sends an API request to Banyan and parses the response. All responses are assumed to be in JSON format.

        .. note:: This method is meant to be called by methods in various resource API classes (e.g.
           :py:class:`PolicyAPI`). You do not need to call :py:meth:`api_request` directly unless you
           are calling an endpoint not currently implemented in this library.

        :param method: HTTP method to execute (e.g. `GET`, `POST`, `HEAD`, `DELETE`, etc.)
        :type method: str
        :param uri: Relative URI or absolute URL endpoint. If a relative URI is provided, it is appended
            to the :py:meth:`api_url` property.
        :type uri: str
        :param params: Optional query string parameters.
        :type params: Dict[str, str]
        :param data: Data to be serialized in JSON format and sent in the body of a `POST` request.
        :param json: Pre-serialized JSON data to be sent in the body of the request. Note that only one of
            `data` or `json` should be supplied.
        :type json: str
        :param headers: Additional headers to be sent with the HTTP request. The library
            automatically adds `Content-Type` and `Authorization` headers.
        :type headers: Dict[str, str]
        :param accept: Content type to expect in the response. If not supplied, the response is assumed
            to be in `application/json` format.
        :type accept: str
        :return: returns the raw Response object from the `Requests` module.
        :rtype: requests.Response
        """
        if not self._access_token:
            self.get_access_token()
        headers = headers or dict()
        headers['Accept'] = accept or self.JSON_TYPE
        if data:
            headers['Content-Type'] = self.JSON_TYPE
        return self._request(method=method, url=uri, params=params, data=data, headers=headers, json=json).json()

    def paged_request(self, method: str, uri: str, params: Dict[str, Any] = None, data: Any = None,
                      json: str = None, headers: Dict[str, str] = None, accept: str = None) -> JsonListOrObj:
        skip = 0
        limit = 1000
        params = params or dict()
        all_results = list()

        while True:
            params['Skip'] = skip
            params['Limit'] = limit
            results = self.api_request(method, uri, params, data, json, headers, accept)
            for key in results.keys():
                logging.debug(f'Looking for {key} in {uri}')
                if key in uri:
                    if len(results[key]) == 0:
                        return all_results
                    all_results.extend(results[key])
                    logging.debug(f'Found {key}, result count = {len(results[key])}, total count = {len(all_results)}')
                    skip += limit

    @property
    def access_token(self) -> str:
        """
        Gets the current access token.
        """
        return self._access_token

    @property
    def refresh_token(self) -> str:
        """
        Gets/sets the refresh token, which is exchanged for an access token that allows us to actually make
        API calls.
        """
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, value: str) -> None:
        self._refresh_token = value
        if self._access_token:
            self._access_token = None

    @property
    def api_url(self) -> str:
        """
        Gets/sets the API server URL. Useful if the URL is not known at instantiation time.
        If an access token has already been obtained, it will be discarded after resetting this property.
        The next API request will result in a new access token being issued.
        """
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = self._normalize_url(value)
        self._access_token = None

    @property
    def services(self) -> ServiceAPI:
        """
        Returns an instance of the :py:class:`ServiceAPI` class, which can be used to manage Banyan services.
        """
        return self._services

    @property
    def policies(self) -> PolicyAPI:
        """
        Returns an instance of the :py:class:`PolicyAPI` class, which can be used to manage Banyan security policies.
        """
        return self._policies

    @property
    def roles(self) -> RoleAPI:
        """
        Returns an instance of the :py:class:`RoleAPI` class, which can be used to manage Banyan roles.
        """
        return self._roles

    @property
    def attachments(self) -> AttachmentAPI:
        """
        Returns an instance of the :py:class:`AttachmentAPI` class, which can be used to query which policies
        are attached to services.
        """
        return self._attach

    @property
    def shields(self) -> ShieldAPI:
        """
        Returns an instance of the :py:class:`ShieldAPI` class, which can be used to get information about
        Banyan Shields.
        """
        return self._shields

    @property
    def netagents(self) -> NetagentAPI:
        """
        Returns an instance of the :py:class:`NetagentAPI` class, which can be used to manage Banyan netagents
        in AccessTier mode or Host mode.
        """
        return self._agents

    @property
    def users(self) -> UserAPI:
        """
        Returns an instance of the :py:class:`UserAPI` class, which can be used to manage Banyan users.
        """
        return self._users

    @property
    def devices(self) -> DeviceAPI:
        """
        Returns an instance of the :py:class:`DeviceAPI` class, which can be used to manage devices such
        as laptops and tablets.
        """
        return self._devices

    @property
    def events(self) -> EventV2API:
        return self._events

    @property
    def audit(self) -> AuditAPI:
        return self._audit


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)

    c = BanyanApiClient(api_server_url='https://gcstage.banyanops.com', refresh_token=os.getenv('BANYAN_REFRESH_TOKEN'),
                        debug=True)
    print(c.get_access_token())
    print(c.services.list())




def get_cloud_vendor() -> str:
    with open('/sys/devices/virtual/dmi/id/board_vendor', 'r') as f:
        vendor = f.read()
        if 'Amazon' in vendor:
            return 'aws'
        elif 'Google' in vendor:
            return 'gcp'
        elif 'Microsoft' in vendor:
            return 'azure'
        else:
            raise RuntimeError(f'unknown cloud vendor: {vendor}')



import fnmatch
from typing import Optional, List

from cryptography import x509
from cryptography.hazmat.primitives.serialization import load_pem_private_key
# noinspection PyUnresolvedReferences
from cryptography.x509.oid import NameOID, ExtensionOID


class CertPair:
    def __init__(self, cert_filename: str, key_filename: str):
        self._cert_file = cert_filename
        self._key_file = key_filename
        with open(self._cert_file, 'rt') as f:
            self._cert = x509.load_pem_x509_certificate(f.read().encode('ascii'))
        with open(self._key_file, 'rt') as f:
            self._key = load_pem_private_key(f.read().encode('ascii'), None)

    def get_common_name(self) -> Optional[str]:
        cn = self._cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)
        if cn:
            return cn[0].value
        return None

    def get_san_dns_names(self) -> List[str]:
        try:
            san: x509.SubjectAlternativeName = self._cert.extensions.get_extension_for_oid(
                ExtensionOID.SUBJECT_ALTERNATIVE_NAME).value
            return san.get_values_for_type(x509.DNSName)
        except x509.extensions.ExtensionNotFound:
            return list()

    def get_names(self) -> List[str]:
        domains = [self.get_common_name()]
        domains.extend(self.get_san_dns_names())
        return domains

    def is_name_match(self, name: str) -> bool:
        for candidate in self.get_names():
            if name == candidate:
                return True
            if '*' in candidate and fnmatch.fnmatch(name, candidate):
                return True
        return False
